@@ -12,11 +12,20 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import numpy as np
+import trimesh
+from copy import deepcopy
 from .constraint import Constraint
-from ...utils import is_string, get_random_point_from_shape
+from ... import random
+from .. import shapes
+from ...utils import is_string, get_random_point_from_shape, \
+    is_array
+from ...transformations import quaternion_matrix
+from ...simulation.properties import Pose
 import collections
 from shapely.geometry import Polygon, LineString, Point, \
-    MultiPoint, MultiPolygon
+    MultiPoint, MultiPolygon, MultiLineString
+from shapely.affinity import affine_transform
 
 
 class WorkspaceConstraint(Constraint):
@@ -90,37 +99,28 @@ class WorkspaceConstraint(Constraint):
     that represent exclusion areas inside the workspace
     """
     _LABEL = 'workspace'
-    _GEOMETRIES = [
-        'line',
-        'area',
-        'volume',
-        'multi_line',
-        'multi_point',
-        'circle',
-        'polygon']
 
-    def __init__(self, geometry=None, frame='world', holes=None):
+    def __init__(self, geometry_type=None, frame='world', holes=None,
+                 pose=None, **kwargs):
         Constraint.__init__(self)
 
-        assert isinstance(
-            geometry, dict), 'Geometry specification must be a dictionary'
-        assert 'type' in geometry, 'No geometry type was specified'
-        assert geometry['type'] in self._GEOMETRIES, \
-            'Invalid workspace geometry, options={}'.format(self._GEOMETRIES)
         assert is_string(frame), \
             'Input frame name must be a string'
 
-        self._geometry_type = geometry['type']
+        self._geometry_type = geometry_type
         self._holes = list()
         self._geometry = None
         self._geometry = self.generate_geometry(
-            self._geometry_type, geometry['description'])
+            self._geometry_type, **kwargs)
 
         if holes is not None:
             for hole in holes:
                 self._holes.append(self.generate_geometry(**hole))
 
         self._frame = frame
+        self._pose = Pose()
+        if pose is not None:
+            self.pose = pose
 
     def __eq__(self, other):
         if not isinstance(other, WorkspaceConstraint):
@@ -144,7 +144,28 @@ class WorkspaceConstraint(Constraint):
         msg += '\t - Type of geometry: {}\n'.format(self._geometry_type)
         return msg
 
-    def generate_geometry(self, type, description):
+    @property
+    def pose(self):
+        return self._pose
+
+    @pose.setter
+    def pose(self, vec):
+        if isinstance(vec, Pose):
+            self._pose = vec
+        else:
+            assert is_array(vec), \
+                'Input pose vector must be iterable'
+            assert len(vec) == 6 or len(vec) == 7, \
+                'Pose must be given as position and Euler angles (x, y, z, ' \
+                'roll, pitch, yaw) or position and quaternions (x, y, z, ' \
+                'qx, qy, qz, qw)'
+            for item in vec:
+                assert isinstance(item, float) or isinstance(item, int), \
+                    'All elements in pose vector must be a float or an integer'
+
+            self._pose = Pose(pos=vec[0:3], rot=vec[3::])
+
+    def generate_geometry(self, type, **kwargs):
         """Generate a `shapely` entity according to the geometry description
         provided. The input `type` containts the name of the geometry to
         be generated and the necessary arguments must be provided in the `dict`
@@ -195,59 +216,114 @@ class WorkspaceConstraint(Constraint):
          are: `line`, `area`, `volume`, `multi_line`, `multi_point`, `circle`
         * `description` (*type:* `dict`): Arguments to describe the geometry
         """
-        if isinstance(description, dict):
-            if 'points' in description:
-                return self._generate_geometry_from_points(type, **description)
-            elif type == 'circle' and \
-                    'center' in description and 'radius' in description:
-                return self._generate_circle(**description)
+        if type == 'area' and 'points' in kwargs:
+            return MultiPoint(
+                [(x[0], x[1]) for x in kwargs['points']]).convex_hull
+        elif type == 'line' and 'points' in kwargs:
+            line = LineString([(x[0], x[1]) for x in kwargs['points']])
+            if 'buffer' in kwargs:
+                assert kwargs['buffer'] > 0, \
+                    'Buffer around line must be greater than 0'
+                return line.buffer(kwargs['buffer'])
             else:
-                raise NotImplementedError()
-        elif type == 'polygon' and \
-                isinstance(description, (Polygon, MultiPolygon)):
-            assert not description.is_empty, 'Polygon is empty'
-            assert description.area > 0, 'Polygon area is zero'
-            return description
+                return line
+        elif type == 'multipoint' and 'points' in kwargs:
+            points = MultiPoint(
+                [(x[0], x[1]) for x in kwargs['points']])
+            if 'buffer' in kwargs:
+                assert kwargs['buffer'] > 0, \
+                    'Buffer around line must be greater than 0'
+                return points.buffer(kwargs['buffer'])
+            else:
+                return points
+        elif type == 'multiline' and 'lines' in kwargs:
+            lines = MultiLineString(kwargs['lines'])
+            if 'buffer' in kwargs:
+                assert kwargs['buffer'] > 0, \
+                    'Buffer around line must be greater than 0'
+                return lines.buffer(kwargs['buffer'])
+            else:
+                return lines
+        elif type == 'circle' and \
+                'center' in kwargs and 'radius' in kwargs:
+            return shapes.circle(**kwargs)
+        elif type == 'polygon' and 'polygon' in kwargs:
+            assert isinstance(kwargs['polygon'], (Polygon, MultiPolygon))
+            assert not kwargs['polygon'].is_empty, 'Polygon is empty'
+            assert kwargs['polygon'].area > 0, 'Polygon area is zero'
+            return kwargs['polygon']
+        elif type == 'box':
+            assert 'size' in kwargs
+            return trimesh.creation.box(extents=kwargs['size'])
+        elif type == 'sphere':
+            assert 'radius' in kwargs
+            assert kwargs['radius'] > 0
+            return trimesh.creation.icosphere(
+                radius=kwargs['radius'])
+        elif type == 'cylinder':
+            assert 'radius' in kwargs
+            assert kwargs['radius'] > 0
+            assert 'length' in kwargs
+            assert kwargs['length'] > 0
+            return trimesh.creation.cylinder(
+                radius=kwargs['radius'],
+                height=kwargs['length'])
         else:
-            raise NotImplementedError()
+            raise NotImplementedError(
+                'Invalid geometry type, provided={}'.format(type))
 
-    def _generate_geometry_from_points(self, geometry_type, points):
-        """For geometries of type `line` and `area`, return the `shapely`
-        entity describing a line or a polygon, respectively.
+    def _apply_transform(self, geometry):
+        mat = quaternion_matrix(self._pose.quat)
+        if isinstance(geometry, trimesh.base.Trimesh):
+            geo = geometry.copy()
 
-        > *Input arguments*
-
-        * `geometry_type` (*type:* `str`): Type of geometry,
-         options are `line` and `area`
-        * `points` (*type:* `list`): List of 3D vectors
-         representing the vertices of the area or the
-         points on the line
-        """
-        if geometry_type == 'line':
-            # Only x and y coordinates are considered for line
-            geometry = LineString([(x[0], x[1]) for x in points])
-        elif geometry_type == 'area':
-            # Only x and y coordinates are considered for polygon area
-            geometry = Polygon([(x[0], x[1]) for x in points])
+            geo.apply_transform(mat)
+            geo.apply_translation(self._pose.position)
+            return geo
         else:
-            raise NotImplementedError()
-        return geometry
+            transform = np.zeros(12)
+            transform[0:3] = mat[0, 0:3].flatten()
+            transform[3:6] = mat[1, 0:3].flatten()
+            transform[6:9] = mat[2, 0:3].flatten()
+            transform[9:12] = self._pose.position
+            return affine_transform(geometry, transform)
 
-    def _generate_circle(self, center, radius):
-        """Return a `shapely.Polygon` representing the circle
-        described by `center` and `radius`.
+    def _compute_random_point_on_geometry(self, geometry):
+        if isinstance(geometry, (Polygon, MultiPolygon)):
+            return Point(
+                get_random_point_from_shape(self.get_geometry()))
+        elif isinstance(geometry, LineString):
+            idx = random.randint(1, len(geometry.coords))
+            start_point = np.array(geometry.coords[idx - 1])
+            vec = np.array(geometry.coords[idx]) - \
+                np.array(geometry.coords[idx - 1])
+            random_point = start_point + random.rand() * vec
+            return Point(random_point)
+        else:
+            raise NotImplementedError('Invalid geometry type={}'.format(
+                type(geometry)))
 
-        > *Input arguments*
+    def _compute_random_point_in_mesh(self, mesh):
+        min_x = mesh.bounds[0, 0]
+        max_x = mesh.bounds[1, 0]
 
-        * `center` (*type:* `list`): 2D or 3D vector
-         representing the center of the circle. For 2D
-         points, the Z component is assumed to be 0.
-        * `radius` (*type:* `float`): Radius of the circle
-        """
-        assert len(center) in [
-            2, 3], 'Center of circle must have 2 or 3 elements'
-        assert radius > 0, 'Radius must be greater than zero'
-        return Point(*center).buffer(radius)
+        min_y = mesh.bounds[0, 1]
+        max_y = mesh.bounds[1, 1]
+
+        min_z = mesh.bounds[0, 2]
+        max_z = mesh.bounds[1, 2]
+        pnt = [
+            random.uniform(min_x, max_x),
+            random.uniform(min_y, max_y),
+            random.uniform(min_z, max_z)
+        ]
+        while not mesh.contains([pnt]):
+            pnt = [
+                random.uniform(min_x, max_x),
+                random.uniform(min_y, max_y),
+                random.uniform(min_z, max_z)
+            ]
+        return Point(pnt)
 
     def get_bounds(self):
         """Return the polygon bounds"""
@@ -255,21 +331,18 @@ class WorkspaceConstraint(Constraint):
 
     def get_random_position(self):
         """Return a random position that belongs to the workspace"""
-        return Point(
-            get_random_point_from_shape(self.get_geometry()))
-        # if self._geometry_type in ['area', 'circle']:
-        #     geo = self.get_geometry()
-        #     min_x, min_y, max_x, max_y = geo.bounds
-        #     pnt = Point(
-        #         random.uniform(min_x, max_x),
-        #         random.uniform(min_y, max_y))
-        #     while not geo.contains(pnt):
-        #         pnt = Point(
-        #             random.uniform(min_x, max_x),
-        #             random.uniform(min_y, max_y))
-        #     return pnt
-        # else:
-        #     return None
+        geometry = self.get_geometry()
+        if isinstance(geometry, (Polygon, LineString)):
+            return self._compute_random_point_on_geometry(geometry)
+        elif isinstance(geometry, (MultiLineString, MultiPolygon)):
+            idx = random.randint(0, len(geometry.geoms))
+            return self._compute_random_point_on_geometry(
+                geometry.geoms[idx])
+        elif isinstance(geometry, MultiPoint):
+            idx = random.randint(0, len(geometry.geoms))
+            return geometry.geoms[idx]
+        else:
+            return self._compute_random_point_in_mesh(geometry)
 
     def contains_point(self, point):
         """Return True if `point` is part of the workspace.
@@ -314,11 +387,27 @@ class WorkspaceConstraint(Constraint):
     def contains_mesh(self, mesh):
         vertices = MultiPoint(mesh.vertices[:, 0:2].tolist())
         geo = self.get_geometry()
-        return geo.contains(vertices.convex_hull)
+
+        if isinstance(geo, (Polygon, MultiPolygon)):
+            return geo.contains(vertices.convex_hull)
+        elif isinstance(geo, (LineString, MultiLineString)):
+            convex_hull = vertices.convex_hull
+            return convex_hull.intersects(geo) or convex_hull.contains(geo)
+        elif isinstance(geo, (MultiPoint)):
+            convex_hull = vertices.convex_hull
+            for point in geo.geoms:
+                if convex_hull.contains(point):
+                    return True
+            return False
+        elif isinstance(trimesh.base.Trimesh):
+            return geo.contains(mesh.vertices)
+
+    def add_hole(self, type, **kwargs):
+        self._holes.append(self.generate_geometry(type, **kwargs))
 
     def get_geometry(self):
         """Return the workspace geometry"""
-        geometry = self._geometry
+        geometry = deepcopy(self._geometry)
         for geo in self._holes:
             geometry = geometry.difference(geo)
-        return geometry
+        return self._apply_transform(geometry)
