@@ -72,6 +72,15 @@ class World(Entity):
     def __str__(self):
         return self.to_sdf().to_xml_as_str(pretty_print=True)
 
+    def __ne__(self, other):
+        result = self.__eq__(other)
+        return not result
+
+    def __eq__(self, other):
+        if not isinstance(other, World):
+            return False
+        return self.to_sdf() == other.to_sdf()
+
     @property
     def physics(self):
         """`pcg_gazebo.simulation.physics.Physics`: Physics engine instance"""
@@ -147,6 +156,10 @@ class World(Entity):
         """`dict`: Model groups"""
         return self._model_groups
 
+    def copy(self):
+        world = World.from_sdf(self.to_sdf())
+        return world
+
     def set_as_ground_plane(self, model_name):
         if '/' not in model_name and 'default' in self._model_groups:
             return self._model_groups['default'].set_as_ground_plane(
@@ -423,8 +436,8 @@ class World(Entity):
         else:
             return self._model_groups[group].light_exists(tag)
 
-    def to_sdf(self, type='world', with_default_ground_plane=True,
-               with_default_sun=True, sdf_version='1.6'):
+    def to_sdf(self, type='world', with_default_ground_plane=False,
+               with_default_sun=False, sdf_version='1.6'):
         """Convert the world description into as `pcg_gazebo` SDF
         element.
 
@@ -678,7 +691,8 @@ class World(Entity):
             line_style='solid',
             alpha=0.5,
             engine='matplotlib',
-            dpi=200):
+            dpi=200,
+            ground_plane_models=None):
         """Plot the mesh footprint projections on the XY plane.
 
         > *Input arguments*
@@ -1266,6 +1280,7 @@ class World(Entity):
         Full name of the exported SDF world file as a `str`
         """
         assert output_dir is not None, 'Output directory cannot be None'
+        from . import is_gazebo_model
 
         if not os.path.isdir(output_dir):
             os.makedirs(output_dir)
@@ -1285,12 +1300,9 @@ class World(Entity):
             return None
 
         for tag in self.models:
-            if self.models[tag].has_mesh:
-                assert models_output_dir is not None, \
-                    'Models export directory cannot be None'
-                assert os.path.isdir(models_output_dir), \
-                    'Models export directory is invalid, dir={}'.format(
-                        models_output_dir)
+            if self.models[tag].has_mesh and \
+                    not is_gazebo_model(
+                        self.models[tag].name, include_custom_paths=True):
                 self.models[tag].to_gazebo_model(
                     sdf_version=sdf_version,
                     output_dir=models_output_dir,
